/* 
    Webcam Lab

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2005 Jason Hunt
    nulluser@gmail.com
    
    file: camera.cpp
*/



#include "core.h"



#include "capture.h"



HWND capture_win = NULL;                // Handle to the capture window

int camera_x_size = 0;         // Size of camera image
int camera_y_size = 0;

color_type *camera_buffer = NULL;       // Buffer for camera data
BITMAPINFO camera_buffer_info;          // Bitmap info for camera

BITMAPINFO camera_image_info;          // Bitmap info for camera


bool running = true;                    // True if we are pulling frames  

bool fast_capture = false;






VideoCapture* vc		= new VideoCapture(frame_callback);
VideoDevice* devices	= vc->GetDevices();
int num_devices			= vc->NumDevices();

void frame_callback(unsigned char* data, int len, int bpp)
{
    memcpy(camera_buffer, data, camera_x_size * camera_y_size * sizeof(color_type));

	frame_ready();
}




/* Setup buffer for camera image */
void camera_setup_buffer( void )
{    
    if (camera_buffer != NULL) free(camera_buffer);    

    camera_buffer = (color_type *) malloc(camera_x_size * camera_y_size * sizeof(color_type));
        
    if (camera_buffer == NULL)
    {            
        add_line("Unable to create camera buffer");
        return;
    }        
    
    /* This sets up the camera buffer bitmap into so we can blt it */

    memset((void *)
    &camera_buffer_info,0, sizeof(camera_buffer_info));
       
    // Set the new Bitmap information
    camera_buffer_info.bmiHeader.biSize = sizeof(camera_buffer_info.bmiHeader);
    camera_buffer_info.bmiHeader.biWidth = camera_x_size;
    camera_buffer_info.bmiHeader.biHeight = camera_y_size; 

    camera_buffer_info.bmiHeader.biPlanes = 1;
    camera_buffer_info.bmiHeader.biBitCount = 24; 
    camera_buffer_info.bmiHeader.biCompression =BI_RGB;
    camera_buffer_info.bmiHeader.biSizeImage = 0;//camera_x_size * camera_y_size  * sizeof(color_type);
    camera_buffer_info.bmiHeader.biClrUsed = 0;
    camera_buffer_info.bmiHeader.biClrImportant = 0;
}
/* End of camera_setup_buffer */





/* Start camera system */
void camera_start( void )
{
  	vc->Select(devices);
  	
  	vc->GetSize(camera_x_size, camera_y_size);
  	
  	
    camera_setup_buffer();
     
    process_start();
    canny_start();
    rc_filter_start();
  	
	vc->Start();

}
/* End of Start camera */



/* Stop camera system */
void camera_stop( void )
{
	vc->Stop();

    if (camera_buffer)
        free(camera_buffer);      
    
    process_stop();
    canny_stop();
    rc_filter_stop();
    
}
/* End of stop_camera */






