/* 
    Webcam Lab

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2005 Jason Hunt
    nulluser@gmail.com
    
    file: capture.h
*/


/* 
    Code copied and adapted from  Limitz - http://paste.rohitab.com/560
*/
#define SKIP_DXTRANS




#pragma comment(lib, "strmiids")


//#define SHOW_DEBUG_RENDERER

/* If you are having the problem you can't open dxtrans.h:
 * Open qedit.h and add this to the start of the file:
 *
 * #ifdef SKIP_DXTRANS
 * #define __IDxtAlphaSetter_INTERFACE_DEFINED__
 * #define __IDxtJpeg_INTERFACE_DEFINED__
 * #define __IDxtKey_INTERFACE_DEFINED__
 * #define __IDxtCompositor_INTERFACE_DEFINED__
 * #endif
 *
 * Also replace the line 
 * #include "dxtrans.h"
 * with:
 * #ifndef SKIP_DXTRANS
 * #include "dxtrans.h"
 * #endif
 */

#ifndef CAPTURE_H
#define CAPTURE_H


#ifndef MAXLONGLONG
#define MAXLONGLONG 0x7FFFFFFFFFFFFFFF
#endif

#ifndef MAX_DEVICES
#define MAX_DEVICES 8
#endif

#ifndef MAX_DEVICE_NAME
#define MAX_DEVICE_NAME 80
#endif

typedef struct
{
	unsigned char R;
	unsigned char G;
	unsigned char B;
} RGB;

typedef void (*VideoCaptureCallback)(unsigned char* data, int len, int bitsperpixel);

class VideoDevice
{
public:
	VideoDevice();
	~VideoDevice();
	
	const char* GetFriendlyName();

private:
	char*			friendlyname;
	WCHAR*			filtername;
	IBaseFilter*	filter;
	friend class	VideoCapture;
};

class VideoCapture
{
public:
	VideoCapture(VideoCaptureCallback callback);
	~VideoCapture();

	VideoDevice* GetDevices();
	int  NumDevices();
	
	void Select(VideoDevice* dev);
	void Start();
	void Stop();
	

	void GetSize(int &xs, int &ys);
	
	

protected:
	void InitializeGraph();
	void SetSourceFilters();
	void SetSampleGrabber();
	void SetNullRenderer();
	void ConnectFilters();

private:
	IFilterGraph2*			graph;
	ICaptureGraphBuilder2*	capture;
	IMediaControl*			control;

	IBaseFilter*			nullrenderer;
	IBaseFilter*			samplegrabberfilter;
	ISampleGrabber*			samplegrabber;

	bool					playing;

	VideoDevice*			devices;
	VideoDevice*			current;

	int						num_devices;
	
    int x_size;
    int y_size;


	class CallbackHandler : public ISampleGrabberCB
	{
	public:
		CallbackHandler(VideoCaptureCallback cb);
		~CallbackHandler();

		void SetMediaType(AM_MEDIA_TYPE* am);

		virtual HRESULT __stdcall SampleCB(double time, IMediaSample* sample);
		virtual HRESULT __stdcall BufferCB(double time, BYTE* buffer, long len);
		virtual HRESULT __stdcall QueryInterface( REFIID iid, LPVOID *ppv );
		virtual ULONG	__stdcall AddRef();
		virtual ULONG	__stdcall Release();
	

	private:
		int						bitpixel;
		VideoCaptureCallback	callback;
   
		
	} *callbackhandler;
};

#endif

