/* 
    Webcam Lab

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2005 Jason Hunt
    nulluser@gmail.com
   
    file: filters.cpp
*/


#include "core.h"

unsigned int size_x = 0;
unsigned int size_y = 0;

/* Vector distence between two colors in a color cube */
inline double distance(color_type a, color_type b)
{
    return(sqrt((a.r-b.r)*(a.r-b.r) +(a.g-b.g)*(a.g-b.g) + (a.b-b.b)*(a.b-b.b)));
}
/* End of distance */


/* Length of a color vector */
double color_size(color_type a)
{
    return(sqrt(a.r*a.r + a.g*a.g + a.b*a.b));
}
/* End of color_size */


/* Draw a box */
void draw_box(unsigned int x1, unsigned int y1, 
              unsigned int x2, unsigned int y2, 
              color_type *m)
{
    // Range check
    if (x1 > size_x-1) x1 = size_x - 1; if (x1 < 0) x1 = 0;
    if (x2 > size_x-1) x2 = size_x - 1; if (x2 < 0) x2 = 0;
    if (y1 > size_y-1) y1 = size_y - 1; if (y1 < 0) y1 = 0;
    if (y2 > size_y-1) y2 = size_y - 1; if (y2 < 0) y2 = 0;
                        
    for (unsigned int y = y1; y < y2 ; y++) 
    {
        unsigned int index = x1 + y * size_x;

        // Horizontal line
        for (unsigned int x = x1; x < x2 ; x++)
            m[index++] = red;
    }
}
/* End of draw box */


/* Compute targetting box */
void target_frame(color_type *s, color_type *d )
{
    unsigned int sum_x = 0, sum_y = 0, num = 0, index = 0;

    // Make sure we have image memory
    if (!s || !d || !last_frame) return;

    // Compute centroid of difference frame
    // With weighted averages

    for (unsigned int y = 0; y < size_y; y++)
        for (unsigned int x = 0; x < size_x; x++)
        {
            double change = distance(s[index], last_frame[index]);
    
            // Apply threshold   50 works
            // Here we are using max_change / 10
    
            if (change < 442/10.0) change = 0;

            unsigned int v = (unsigned int) change;

            if (d_frame) // If we need to display the difference frame
            {
                // Readjust the value into the range [0, 255]
   
                unsigned int tmp = (v * 255) / 442;

                if (tmp > 255) tmp = 255;   // clamp
                
                d[index].g = (unsigned char)tmp;
                d[index].r = 0;        
                d[index].b = 0;
            }

            sum_x += x*v;   // Add weighted point to sum average
            sum_y += y*v;
            
            num+=v;   

            index++;
        }
  
    // Done with image, save to old image
    memcpy(last_frame, s, size_x*size_y * sizeof(color_type));   
  
    // Get origional frame
    if (!d_frame)
        memcpy(d, s, sizeof(color_type)*size_x*size_y);

    // Targe if we have enough movement 
    if (num > 10000)
    {
        // Calculate the centroid of the difference frame
        unsigned int center_x = sum_x / num;
        unsigned int center_y = sum_y / num;

        index = center_x;

        // Draw lines
        for (unsigned int y = 0; y < size_y; y++, index+=size_x)
            d[index].g = 255;        

        // get index for horizontal line
        index = center_y * size_x;

        for (unsigned int x = 0; x < size_x; x++)
            d[index++].g = 255;        

        // Draw targetting box
        draw_box(center_x-5,center_y-5, center_x+5, center_y+5, d);        
    }
}
/* End of target frame */


/* Compute the average of the last two frames */
void average_frame( color_type *s, color_type *d )
{

    for (unsigned int i = 0; i < SIZE; i++)
    {
        d[i].r = ((int)s[i].r + (int)last_frame[i].r) / 2;
        d[i].g = ((int)s[i].g + (int)last_frame[i].g) / 2;
        d[i].b = ((int)s[i].b + (int)last_frame[i].b) / 2;

    }
            
    memcpy(last_frame, s, DATA_SIZE);
}
/* End of average frame */


/* Find zero crossings  */
void erode_frame( color_type *s, color_type *d, double t1)
{
    unsigned int index = 0; // skip first line
    
    // skip first line, copy form origional
    for (unsigned int x = 0; x < size_x; x++, index++)
       d[index] = save_frame[index];
    
    for (unsigned int y = 1; y < size_y - 1; y++)
    {        
        // Skip first pixel, copy from original image
        d[index] = save_frame[index];
        index++;
        
        for (unsigned int x = 1; x < size_x - 1; x++)
        {
             // point to the line above and one to the left
            unsigned int t = index;
          
            bool edge = false;

            // Check for passing over a zero of the laplacian
            // Do not mark two pixels in a row

            // horizontal edge
           if ((color_size(s[t-1]) > t1) && (color_size(s[t+1]) < t1)  && (d[t-1].r!=255)) edge = true;
           
            // verdical edge
           if ((color_size(s[t-size_x]) > t1) && (color_size(s[t+size_x]) < t1) && (d[t-size_x].r!=255)) edge = true;


            // verdical edge
           if ((color_size(s[t-size_x-1]) > t1) && (color_size(s[t+size_x+1]) < t1) && (d[t-size_x-1].r!=255)) edge = true;

            // verdical edge
           if ((color_size(s[t+size_x+1]) > t1) && (color_size(s[t+size_x+1]) < t1) && (d[t+size_x-1].r!=255)) edge = true;

        
            if (edge)
                d[index] = red;                 // Mark as edge
            else
                d[index] = save_frame[index];   // Use original image value

            index++; 
        }

        d[index] = save_frame[index];
        index++;
    }

    for (unsigned int x = 0; x < size_x; x++, index++)
       d[index] = save_frame[index];
    
}
/* End of erode_frame */



/* Preform fast edge detection */
void edge_detect_frame( color_type *s, color_type *d)
{
    color_type current, right, below;

    unsigned int index = 0;

    const int limit = 16;

    for (int y = 0; y < size_y-1; y++)
    {
        for (int x = 0; x < size_x-1; x++)
        {
            // Calcuate gradient vector using the right pixels the pixel below
            current = s[index];
            right = s[index+1];
            below = s[index + size_x];

           /* if (threshold)
            {
                
                if (distance(current, right) > limit || 
                    distance(current, below) > limit)
                     d[index++] = get_color(255, 255, 255); 
                else
                     d[index++] = get_color(0, 0, 0);
            }
            else*/
            {
                // approximate distance
        
                int r = 5*(abs(current.r - right.r) + abs(current.r - below.r));
                int g = 5*(abs(current.g - right.g) + abs(current.g - below.g));
                int b = 5*(abs(current.b - right.b) + abs(current.b - below.b));

                // Clamp colors
                if (r > 255) r = 255;
                if (g > 255) g = 255;
                if (b > 255) b = 255;
        
                d[index].r = r;
                d[index].g = g;
                d[index].b = b;

                index++;
            }
        }
        index++; // Skip over unused right edge
    }

}
/* End of edge detect frame */


/* This can apply the x and y filters seperatly */
// s: source buffer
// d: destination buffer
// x_filter: 2d filter for x direction
// y_filter: 2d filter for y direction
// size:  filter is size*size

void general_filter_frame( color_type *s, color_type *d, 
                           const double *x_filter, const double *y_filter, 
                           unsigned int size, double scale)
{
    unsigned int index = size_x; // skip first line

    for (int y = 1; y < size_y-1; y++)
    {
        index++;
        for (int x = 1; x < size_x-1; x++)
        {
            double x_sum_r = 0;
            double x_sum_g = 0;
            double x_sum_b = 0;
          
            double y_sum_r = 0;
            double y_sum_g = 0;
            double y_sum_b = 0;
          
            // point to the line above and one to the left
            unsigned int t = index - size_x - 1;
          
            unsigned int f = 0;
            // preform convolution
             for (int i = 0; i < 3; i++)
             {
                 for (int j = 0; j < 3; j++)
                 {                                             
                     x_sum_r += scale* (double)s[t].r * x_filter[f];
                     x_sum_g += scale* (double)s[t].g * x_filter[f];
                     x_sum_b += scale* (double)s[t].b * x_filter[f];

                     y_sum_r += scale* (double)s[t].r * y_filter[f];
                     y_sum_g += scale* (double)s[t].g * y_filter[f];
                     y_sum_b += scale* (double)s[t].b * y_filter[f];
                 
                     t++;  f++;                      
                }
                
                t += size_x - 3; // next line down, and 3 to left
            }

            int r = (int)(fabs(x_sum_r) + fabs(y_sum_r)) << 3;
            int g = (int)(fabs(x_sum_g) + fabs(y_sum_g)) << 3;
            int b = (int)(fabs(x_sum_b) + fabs(y_sum_b)) << 3;                            
                              
            if (r > 255) r = 255;  if (r < 0) r = 0;
            if (g > 255) g = 255;  if (g < 0) g = 0;
            if (b > 255) b = 255;  if (b < 0) b = 0;
         
            // set output color                         
            d[index].r = r;
            d[index].g = g;
            d[index].b = b;
                    
            index++; 
        }
        index++; // Skip over unused right edge
    }

}
/* End of general filter */


/* Perform a symettric transform */
void transform_frame( color_type *s, color_type *d, const double *filter, 
                      unsigned int size, double scale, double base)
{
    unsigned int skip = (size-1) / 2; // buffer zone around edges to skip

    unsigned int index = size_x * skip; // skip first line

    for (int y = skip; y < size_y - skip; y++)
    {        

        // these cannot be processed because of the mask size
        for (int x = 0; x < skip; x++)
            d[index++] = black;
        
        for (int x = skip; x < size_x - skip; x++)
        {
            double sum_r = 0;
            double sum_g = 0;
            double sum_b = 0;
 
            // point to the line above and one to the left
            unsigned int t = index - size_x * skip - skip ;
          
            unsigned int f = 0; // index into the filter
          
            // preform convolution
             for (int i = 0; i < size; i++)
             {
                 for (int j = 0; j < size; j++)
                 {
                     sum_r += (double)s[t].r * filter[f];
                     sum_g += (double)s[t].g * filter[f];
                     sum_b += (double)s[t].b * filter[f];

                     t++;                        
                     f++;
                }
                
                t += size_x - size; // next line down, and 3 to left
            }            
        
            int r = (int)(sum_r * scale + base);
            int g = (int)(sum_g * scale + base);
            int b = (int)(sum_b * scale + base);
        
            if (r > 255) r = 255; if (r < 0) r = 0;
            if (g > 255) g = 255; if (g < 0) g = 0;
            if (b > 255) b = 255; if (b < 0) b = 0; 
         
             // set Output color                
            d[index].r = r; 
            d[index].g = g;
            d[index].b = b;

            index++; 
        }


        // these cannot be processed because of the mask size
        for (int x = 0; x < skip; x++)
            d[index++] = black;

//        index+=skip; // Skip over unused right edge
    }

}
/* End of transform_frame */


/* Comparison function for quick sort */
int compare(const void *a, const void * b)
{
    return(color_size(*(color_type*)a) > color_size(*(color_type*)b));
}
/* End of compare */


/* Perform a symettric transfor */
void median_frame( color_type *s, color_type *d)
{
    unsigned int size = 5;

    unsigned int skip = (size-1) / 2; // buffer zone around edges to skip

    unsigned int index = size_x * skip; // skip first line

    color_type color_list[size * size];

    for (int y = skip; y < size_y - skip; y++)
    {
        index+=skip;
        for (int x = skip; x < size_x - skip; x++)
        {
            // point to the line above and one to the left
            unsigned int t = index - size_x - skip;
    
            unsigned l = 0;
              
            // preform convolution
             for (int i = 0; i < size; i++)
             {
                 for (int j = 0; j < size; j++)
                     color_list[l++] = s[t++];
                                 
                t += size_x - size; // next line down, and 3 to left
            }
        
            //  SLOW!!!
            qsort(color_list, size, sizeof(color_type), compare);
                 
            // set output color                
            d[index++] = color_list[(size+1)/2]; 
        }
        index+=skip; // Skip over unused right edge
    }
}
/* End of median_frame */


/* Apply a threshood to a frame */
void threshold_frame( color_type *s, color_type *d, double t)
{
    unsigned int num = size_x * size_y;
        
    while(num--)
    {
        if (color_size(*s) > t)
            *d = black;
        else
            *d = white;
    
        s++;  d++;
    }
}
/* End of threshold_frame */




void reduceedge_frame(color_type *s, color_type *d, double t)
{
    
    double t2 = 0.75 * t;
    
    int phase = 0;
    
    // track horizontal edges
    for (int y = 0; y < size_y; y++)    
    {
        double start_size = color_size(s[y*size_x]);

        d[y*size_x] = black;; // set the pixel we skipped ofer to black

        for (int x = 1; x < size_x; x++) //  skip first pixel
        {

            double cur = color_size(s[x+y*size_x]);
            
            if ((start_size > t && cur < t) || (start_size < t && cur > t))
            {
                start_size = cur;

              //  if (phase++ % 2)
                    d[x+y*size_x] = white;                
              //  else
                 //   d[x+y*size_x] = black;                
            }
            else
               d[x+y*size_x] = black;


        }
    }
    
    // track horizontal edges
    for (int x = 0; x < size_x; x++)    
    {
        double start_size = color_size(s[x]);

       // d[y*size_x] = black;; // set the pixel we skipped ofer to black

        for (int y = 1; y < size_y; y++) //  skip first pixel
        {

            double cur = color_size(s[x+y*size_x]);
            
            if ((start_size > t && cur < t) || (start_size < t && cur > t))
            {
                start_size = cur;

               // if (phase++ % 2)
                    d[x+y*size_x] = white;                
                //else
                //    d[x+y*size_x] = black;                
            }
           // else
            //   d[x+y*size_x] = black;


        }
    }

}


void combo_edgedetect_frame( color_type *s, color_type *d, double t1)
{
    unsigned int size = 5; // size of laplace

    unsigned int skip = (size-1) / 2; // buffer zone around edges to skip

    unsigned int index = size_x * skip; // skip first line

    for (int y = skip; y < size_y - skip; y++)
    {        

        // these cannot be processed because of the mask size
        for (int x = 0; x < skip; x++)
            d[index++] = black;
        
        for (int x = skip; x < size_x - skip; x++)
        {
            // calculate the grident magnitude with the sobel filters
            
            double x_sum = 0;
            double y_sum = 0;
          
            // point to the line above and one to the left
            unsigned int t = index - size_x - 1;
          
            unsigned int f = 0;
            // preform convolution
             for (int i = 0; i < 3; i++)
             {
                 for (int j = 0; j < 3; j++)
                 {                        
                     x_sum += color_size(s[t]) * sobel_x[f] * 1/3.0;
                     y_sum += color_size(s[t]) * sobel_y[f]* 1/3.0;                     
                 
                     t++;  f++;                      
                }
                
                t += size_x - 3; // next line down, and 3 to left
            }

            double grad_size = fabs(x_sum) + fabs(y_sum);             
            double grad_dir = 0;
            
            if (x_sum != 0)
                grad_dir = (atan(y_sum / x_sum) + M_PI / 2.0) / M_PI * 255;
            
            
            unsigned int c = (int) (   grad_dir);

            if (c > 255) c = 255;
            if (c < 0) c = 0;
            

            
            d[index].r = c;
            d[index].g = c;
            d[index].b = c;
            
            
            
          /*  double sum_r = 0;
            double sum_g = 0;
            double sum_b = 0;
 
            // point to the line above and one to the left
             t = index - size_x * skip - skip ;
          
             f = 0; // index into the filter


            // Apply the laplace filter
          
              double l_sum = 0;
          
            // preform convolution
             for (int i = 0; i < size; i++)
             {
                 for (int j = 0; j < size; j++)
                 {
                        
                        l_sum += color_size(s[t]) * laplace5[f];
//                     sum_r += (double)s[t].r * laplace5[f];
//                     sum_g += (double)s[t].g * laplace5[f];
//                     sum_b += (double)s[t].b * laplace5[f];

                     t++;                        
                     f++;
                }
                
                t += size_x - size; // next line down, and 3 to left
            }            
        */
            /*int r = (int)(sum_r * 5 + 128);
            int g = (int)(sum_g * 5 + 128);
            int b = (int)(sum_b * 5 + 128);
        
            if (r > 255) r = 255; if (r < 0) r = 0;
            if (g > 255) g = 255; if (g < 0) g = 0;
            if (b > 255) b = 255; if (b < 0) b = 0; */
         
       /*  unsigned int c = (l_sum+128) * grad_size/255.0;

        if (c > 255) c = 255; 
        if (c < 0) c = 0;

         
         if (grad_size > 0)//t1)
         {
         
             // set Output color                
            d[index].r = c; 
            d[index].g = c;
            d[index].b = c;
        }
        else
            d[index] = black;*/
  
            index++; 
        }


        // these cannot be processed because of the mask size
        for (int x = 0; x < skip; x++)
            d[index++] = black;

    }


    
}

    
        

















