/* 
    Webcam Lab

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2005 Jason Hunt
    nulluser@gmail.com
   
    file: filters.h
*/


#ifndef FILTERS_H
#define FILTERS_H


#define SIZE (size_x * size_y)
#define DATA_SIZE (size_x * size_y * sizeof(color_type))
                             
// Symmetric filters

const double gauss_blur[25] = {2,  4,  5,  4,  2,
                               4,  9, 12,  9,  4,
                               5, 12, 15, 12,  5,
                               4,  9, 12,  9,  4,
                               2,  4,  5,  4,  2};

// laplacian
const double average[9] = { 1,  1,  1, 
                            1,  1,  1,
                            1,  1,  1};

// laplacian
const double laplace[9] = { -1, -1, -1, 
                            -1,  8, -1,
                            -1, -1, -1};

const double laplace5[25] =  { 0,  0,  1,  0,  0,
                               0,  1,  2,  1,  0,
                               1,  2, -16, 2,  1,
                               0,  1,  2,  1,  0,
                               0,  0,  1,  0,  0};
    
    
const double sharpness[9] = {0,  -1, 0, 
                            -1, 5, -1,
                             0,  -1, 0};
           
  
const double embossing[9] = {2,  0, 0, 
                             0, -1, 0,
                             0,  0, -1};    

// Seperable filters 
const double sobel_x[9] = {-1, 0, 1,
                           -2, 0, 2, 
                           -1, 0, 1};

const double sobel_y[9] = {-1, -2, -1,
                            0,  0,  0,
                            1,  2,  1};           


// Exported functions

double color_size(color_type a);

void threshold_frame( color_type *s, color_type *d, double t);

void median_frame( color_type *s, color_type *d);

void transform_frame( color_type *s, color_type *d, const double *filter, 
                      unsigned int size, double scale, double base);

void general_filter_frame( color_type *s, color_type *d, 
                           const double *x_filter, const double *y_filter, 
                           unsigned int size, double scale);

void erode_frame( color_type *s, color_type *d, double t1);
                           
void edge_detect_frame( color_type *s, color_type *d);

void target_frame(color_type *s, color_type *d );

void reduceedge_frame(color_type *s, color_type *d, double t);
                                                         
void average_frame( color_type *s, color_type *d );                             
                             
void combo_edgedetect_frame( color_type *s, color_type *d, double t);

extern unsigned int size_x;
extern unsigned int size_y;

#endif                             
                       
                   
                             
                             
